import numpy as np
import pandas as pd
import itertools
from scipy.stats import kendalltau, rankdata

from .utils_legacy import *


def method_C(id_evaluation, binary_matrix_current, choices_participants):
    """ Calculate the ranking from participants from their choices
    """
    points_projects, flag_error = get_allocated_points(id_evaluation, choices_participants)

    # Calculate value ranking according to points allocated and given binary score matrix.
    # Note: In some cases the participants did not distribute the 100 points, but a smallar portion of it
    weighted_values = np.sum(np.transpose(points_projects) * binary_matrix_current, axis=1)
    #We use the negative (-) symbol on the line below to rank as 1st the HIGHER value and last the LOWER value
    ranking_values  = rankdata(-weighted_values, method='min')

    return weighted_values, flag_error, points_projects, ranking_values


def equivalence_rate(weighted_values_er, motivation_flag, id_eval, result):
    """ Fill result with number of "~" (ties) we have in the ranking.
    """
    er = len(weighted_values_er) - len(pd.unique(weighted_values_er))
    result[f'ER_{motivation_flag}'][float(id_eval)] = er

    return result


def method_TB(weighted_values, id_evaluation, values_name, motivations_df):
    """ Break ties from already calculated rankings (weighted_values).
    """
    values_name_list = values_name['Nederlands'].to_list()
    weighted_values_updated = weighted_values.copy()
    increase_value = 0.5
    cont_ties = 0

    for p in range(1, 6+1): #For all projects
        #For some cases there is a participant that made choices (20200525_5_LK.xlsx) but provided no justification.
        # Empirically it seems to be the case that if they are in one motivation (q1.xlsx), they are also not on the
        # other (q2.xlsx ... q6.xlsx)
        if motivations_df[p][values_name_list][motivations_df[p].id == id_evaluation].empty:
            continue

        # Get the value(s) supported by the motivation.
        index = motivations_df[p][values_name_list][motivations_df[p].id == id_evaluation] == 1
        supported_value_mot = []
        for i in range(len(list(index))):
            if index.iloc[:,i].bool() == 1:
                supported_value_mot.append(i)

        # If those value(s) have a score equal to another value of the user, add increase_value to them.
        for i in range(len(supported_value_mot)):
            if check(weighted_values_updated[[supported_value_mot[i]]] == weighted_values, 2):
                #I compare to the original ranking (weighted_values). Otherwise I would be giving priority to first projects (e.g. proj 1)
                weighted_values_updated[[supported_value_mot[i]]] += increase_value
                cont_ties += 1

    return weighted_values_updated, cont_ties


def method_M(id_evaluation, values_name, motivations_df):
    """ Calculate the number of values mentioned in the motivations (method M).
    """
    values_name_list = values_name['Nederlands'].to_list()
    weighted_values = np.array([0, 0, 0, 0, 0])
    weighted_values_updated = weighted_values.copy()
    value_per_cite = 1
    cont_values = 0

    for p in range(1, 6+1): #For all projects
        #For some cases there is a participant that made choices (20200525_5_LK.xlsx) but provided no justification.
        # Empirically it seems to be the case that if they are in one motivation (q1.xlsx), they are also not on the
        # other (q2.xlsx ... q6.xlsx)
        if motivations_df[p][values_name_list][motivations_df[p].id == id_evaluation].empty:
            continue

        # Get the value(s) supported by the motivation.
        index = motivations_df[p][values_name_list][motivations_df[p].id == id_evaluation] == 1
        supported_value_mot = []
        for i in range(len(list(index))):
            if index.iloc[:,i].bool() == 1:
                supported_value_mot.append(i)
        # If those value(s) have a score equal to another value of the user, add increase_value to them.

        # print('supported_value_mot')
        # print(supported_value_mot)
        # print('len(supported_value_mot)')
        # print(len(supported_value_mot))
        # print('range(len(supported_value_mot))')
        # print(range(len(supported_value_mot)))
        # print('range(len(supported_value_mot))')

        for i in range(len(supported_value_mot)):
            # if check(weighted_values_updated[[supported_value_mot[i]]] == weighted_values, 2):
            weighted_values_updated[[supported_value_mot[i]]] += value_per_cite
            cont_values += 1

    # print(weighted_values_updated)
    ranking_values  = rankdata(-weighted_values_updated, method='min')

    return weighted_values_updated, cont_values, ranking_values


def calc_sptd_motivation(id_evaluation, cont_sptd_mot, values_name, motivations_df, sptd_motivation, flag_sptd_mot):
    """ For each project, when there is a supported value, add participant ID and value number
        to sptd_motivation[project_number]. Then increase cont_sptd_mot count for that project.
    """
    values_name_list = values_name['Nederlands'].to_list()

    for p in range(1, 6+1): #For all projects
        # For some cases there is a participant that made choices (20200525_5_LK.xlsx) but provided no justification.
        # Empirically it seems to be the case that if they are in one motivation (q1.xlsx), they are also not on the
        # other (q2.xlsx ... q6.xlsx)
        if motivations_df[p][values_name_list][motivations_df[p].id == id_evaluation].empty:
            continue

        index = motivations_df[p][values_name_list][motivations_df[p].id == id_evaluation] == 1

        # If there is a supported values, add participant ID and value number to sptd_motivation[project_number]
        for i in range(len(list(index))):
            if index.iloc[:,i].bool() == 1:
                sptd_motivation[p].at[cont_sptd_mot[p-1],'ID'] = id_evaluation
                sptd_motivation[p].at[cont_sptd_mot[p-1],'VM'] = i
                cont_sptd_mot[p-1] += 1
                flag_sptd_mot = True

    return sptd_motivation, cont_sptd_mot, flag_sptd_mot


def inconsistencies_motivations_motivations(id_eval,binary_matrix_projects, sptd_motivation):
    """ For each project, check if there is a conflict between a value annotated from the movitation and the other project-supported values,
        with values annotated from all other projects.
    """
    binary_matrix_projects_individual = binary_matrix_projects.copy()

    for p_from in range(1, 6+1):
        # Get values supported by motivations and by choices.
        aux_ID = np.where(sptd_motivation[p_from].ID == id_eval)[0]
        values_M = list(sptd_motivation[p_from].VM[aux_ID].dropna()) #Disconsider the NaN
        values_C = list(np.where(binary_matrix_projects[:, p_from-1])[0])

        if not len(values_M):
            continue

        # Remove values_M from values_C (so to avoid comparisions between the same value).
        for M in values_M:
            if values_C.count(M) > 0: # check if M is in values_C
                values_C.remove(M)

        for p_to in range(1, 6+1):
            if p_to == p_from:
                continue

            # Get values supported by motivations and by choices.
            aux_ID = np.where(sptd_motivation[p_to].ID == id_eval)[0]
            values_comp_M = list(sptd_motivation[p_to].VM[aux_ID].dropna()) #Disconsider the NaN
            values_comp_C = list(np.where(binary_matrix_projects[:, p_to-1])[0])

            # Remove values_M from values_C (so to avoid comparisions between the same value).
            if len(values_comp_M):
                for M in values_comp_M:
                    if values_comp_C.count(M) > 0: # check if M is in values_C
                        values_comp_C.remove(M)

            # Find if values supported in values_C are in values_comp_M
            for C in values_C: #values_C refer to the original project
                if values_comp_M.count(C) > 0: #Check if C is in values_comp_M
                    for M in values_M:
                        if values_comp_C.count(M) > 0: #Check if M is in values_comp_C, i.e. if there is a conflict between both projects
                            binary_matrix_projects_individual[C, p_from-1] = 0

    return binary_matrix_projects_individual


def inconsistencies_choices_motivations(id_eval, binary_matrix_projects, weighted_values_ind, sptd_motivation):
    """ Solve inconsistencies between values supported by motivations and by choices.
        Values supported by motivations should always be higher than those supported by choices;
        if it is not the case, set to 0 the correspondent value supported by the choice (in the binary matrix).
    """
    binary_matrix_projects_individual = binary_matrix_projects.copy()

    for p in range(1, 6+1):
        # Get values supported by motivations and by choices.
        aux_ID = np.where(sptd_motivation[p].ID == id_eval)[0]
        values_M = list(sptd_motivation[p].VM[aux_ID].dropna()) #Disconsider the NaN
        # #sptd motivation has index project 1 = p[1]; while binary_matrix_project has project 1 = binary_ma..[:,p-1]
        values_C = list(np.where(binary_matrix_projects[:, p-1])[0])

        if not len(values_M):
            continue

        # Remove values_M from values_C (so to avoid comparisions between the same value).
        for M in values_M:
            if values_C.count(M) > 0: #Check if M is in values_C
                values_C.remove(M)

        # If there is inconsistency between one value_M and value_C, set that value_C to 0.
        for M in values_M:
            for C in values_C:
                if weighted_values_ind[M] <= weighted_values_ind[C]: #Inconsistency M, values_C
                    #M is expected to be greater than C. Otherwise, there is an inconsistency.
                    binary_matrix_projects_individual[C, p-1] = 0

    return binary_matrix_projects_individual


def calculate_kendall_tau(user_id,methods,complete_results):
    methods_pairs = itertools.combinations(methods, 2) # all combinations of the 9 methods
    tau_result = pd.DataFrame(index=methods, columns=methods)

    taus = [] # list of tuples
    for method_1, method_2 in methods_pairs:
    #     print('PAIR: ',method_1,' x ' ,method_2)
        aux = kendalltau(   rankdata(-complete_results[user_id][method_1],method='min'),
                            rankdata(-complete_results[user_id][method_2],method='min'))
        # taus.append(aux)
        # print(aux.pvalue)
        tau_result.loc[method_1][method_2] = aux[0]

    # correlations = [tau[0] for tau in taus]
    return tau_result


def kendall_tau_pair(rank1,rank2):
    tau = kendalltau( rank1,rank2)
    return tau


def pairwise_comp(id_eval,complete_results,methods,pairwise_results,list_pairwise_results,cont_changes):
    ## Check the pairwise differences in the ranking across all id and methods


    # for id_eval in choices_participants.id:
    if any(complete_results.columns == id_eval):
        for m in methods:
            rank_m = rankdata(-(complete_results[id_eval][m]), method='min')
            aux = np.empty((len(rank_m), len(rank_m)))
            aux[:] = np.NaN

            for r in range(0, len(rank_m)):
                for c in range(r, len(rank_m)):
                    if rank_m[r] > rank_m[c]:
                        aux[r][c] = 1
                    elif rank_m[r] == rank_m[c]:
                        aux[r][c] = 0
                    elif rank_m[r] < rank_m[c]:
                        aux[r][c] = -1
            pairwise_results[id_eval, m] = aux

    # Check the differences between the methods

    # for id_eval in choices_participants.id:
    if any(complete_results.columns == id_eval):
        list_pairwise_results[id_eval] = []
        #         print(id_eval)
        methods_pairs = itertools.combinations(methods, 2)  # all combinations of the 9 methods
        for method_1, method_2 in methods_pairs:  # m2,m1 to match the structure of the df
            #             print(method_1)
            #             print(method_2)
            ut = np.triu(pairwise_results[id_eval, method_1]) - np.triu(pairwise_results[id_eval, method_2])
            #             print(ut)
            # Upper triangle of the differences
            diff_pairwise_results = np.where(np.abs(ut))
            # location of the differences
            #             print(diff_pairwise_results[0])
            #             print(diff_pairwise_results[1])
            #             print(range(0,len(diff_pairwise_results[0])-1))
            for l in range(0, len(diff_pairwise_results[0])):
                #                 print('[',diff_pairwise_results[0][l],',',diff_pairwise_results[1][l],']')
                if not ([diff_pairwise_results[0][l], diff_pairwise_results[1][l]] in list_pairwise_results[id_eval]):
                    # Checks if element is not already on the list
                    list_pairwise_results[id_eval].append(
                        [diff_pairwise_results[0][l], diff_pairwise_results[1][l]])
                    # If a change happens more often among different
                    cont_changes += 1
    return pairwise_results, list_pairwise_results, cont_changes


def method_C_MO_MC_TB(id_eval, binary_matrix_projects, choices_participants, values_name, motivations_df):
    weighted_values_C, _, _, _ = method_C(id_eval, binary_matrix_projects, choices_participants)

    sptd_motivation = {}
    for p in range(1, 6 + 1):  # For all 6 projects
        df = pd.DataFrame(columns=['ID', 'VM'])
        sptd_motivation[p] = df.copy()

    binary_matrix_projects_MM = inconsistencies_motivations_motivations(id_eval, binary_matrix_projects,
                                                                        sptd_motivation)
    weighted_values_MM, _, _, _ = method_C(id_eval, binary_matrix_projects_MM, choices_participants)

    binary_matrix_projects_MM_MC = inconsistencies_choices_motivations(id_eval,binary_matrix_projects_MM,
                                                                       weighted_values_MM, sptd_motivation)
    weighted_values_MM_MC, _, _, _ = method_C(id_eval, binary_matrix_projects_MM_MC, choices_participants)

    weighted_values_MM_MC_TB, cont_ties = method_TB(weighted_values_MM_MC,id_eval, values_name, motivations_df)

    ranking_values_MM_MC_TB  = rankdata(-weighted_values_MM_MC_TB, method='min')

    return weighted_values_MM_MC_TB, cont_ties, ranking_values_MM_MC_TB
