import numpy as np
import pandas as pd
import math

value_list = ['v0', 'v1', 'v2', 'v3', 'v4']

methods = ['ER_C','ER_C_TB','ER_C_MC','ER_C_MM','ER_C_MM_MC_TB','ER_M']


def import_datasets(threshold_value_project_participant = 20, threshold_value = 250, exclude_value = {'Nederlands': ['nood'], 'English': ['necessity']}):
	""" Load datasets.
	"""
	xls = pd.ExcelFile('./value_profiles/data/20200525_5_LK.xlsx')
	choices_participants = pd.read_excel(xls, '20200519_4e sessie LK')

	values_projects = pd.read_csv('./value_profiles/data/20200618_all_values_english-removed_necessity.csv')

	df = {'Nederlands': ['ambitie','eerlijk','gelijk','inspraak','kennis','kosten','land',
    'leef','leiding','lokaal','markt','nood','onder','samen','tech',
    'verant','vertrouw','zelf'], 'English': ['ambition','honesty','equality',
    'participation','knowledge','(cost)effectiveness','respect for nature and landscape',
    'liveability','leadership','local (organisation)','(anti)market segment',
    'necessity','support','cooperation','technological (innovation)','responsibility','trust','self determination']}

	#Exclude the values from the exception list
	for i in range(0,len(exclude_value['Nederlands'])):
		df['Nederlands'].remove(exclude_value['Nederlands'][i])

	for i in range(0,len(exclude_value['English'])):
		df['English'].remove(exclude_value['English'][i])

	#Filter the values based on the number of annotations
	selected_values = values_projects.iloc[:,2:8] [np.sum(np.array(values_projects.iloc[:,2:8]), axis=1)>threshold_value]
	df['Nederlands'] = list( df['Nederlands'][i] for i in list(selected_values.index.values) )
	df['English'] = list( df['English'][i] for i in list(selected_values.index.values) )

	binary_matrix_projects = np.array(values_projects.iloc[list(selected_values.index.values) ,2:8] > threshold_value_project_participant)*1

	#Import motivations
	motivations_files = ['q1', 'q2', 'q3', 'q4', 'q5', 'q6']
	motivations_df = {}

	for index, file_name in enumerate(motivations_files):
		xls = pd.ExcelFile(f'./value_profiles/data/{file_name}.xlsx')
		df2 = pd.read_excel(xls, 'Data')
		motivations_df[index + 1] = df2.copy()

	values_name = pd.DataFrame(data=df)

	return choices_participants, binary_matrix_projects, motivations_df, values_name, selected_values


def get_allocated_points(id_evaluation, choices_participants):
	# Collect points assigned by the user to each project.
    points_projects = [choices_participants.project_228[choices_participants.id == id_evaluation],
            choices_participants.project_230[choices_participants.id == id_evaluation],
	        choices_participants.project_231[choices_participants.id == id_evaluation],
            choices_participants.project_232[choices_participants.id == id_evaluation],
            choices_participants.project_233[choices_participants.id == id_evaluation],
            choices_participants.project_234[choices_participants.id == id_evaluation]]

    # Convert points to float.
    try:
        points_projects = np.array([float(x) for x in points_projects])
    except Exception as e:
        print('ERROR')
        print(e)

    flag_error = 0
    if any([math.isnan(x) for x in points_projects]):
        print('NaN ERROR')
        flag_error = 1
        return "", flag_error, "", ""
    
    return points_projects, flag_error


def calc_binary_matrix(threshold_value, threshold_value_project_participant):
	values_projects = pd.read_csv('./value_profiles/data/20200618_all_values_english-removed_necessity.csv')
	selected_values = values_projects.iloc[:,2:8] [np.sum(np.array(values_projects.iloc[:,2:8]), axis=1)>threshold_value]
	binary_matrix_projects = np.array(values_projects.iloc[list(selected_values.index.values) ,2:8] > threshold_value_project_participant)*1
	return binary_matrix_projects


def check(it, num):
    it = iter(it)
    return all(any(it) for _ in range(num))


def make_results_df(ids):
	""" Create DataFrame to store value scores.
		Index=[participant_ids], Columns = [methods, values].
	"""
	# Create NaN matrix of data.
	data = np.empty((len(ids), len(methods) * len(value_list)))
	data[:] = np.NaN

	# Create column indices.
	values_indices  = value_list * len(methods)
	methods_indices = [m for method in methods for m in [method] * len(value_list)]

	# Create columns as MultiIndex and index as participants' ID.
	columns = pd.MultiIndex.from_tuples(list(zip(methods_indices, values_indices)), names=['method', 'value'])
	index   = pd.Index(ids, name='participant_id')

	# Create multi-dimensional DataFrame.
	df = pd.DataFrame(data=data, columns=columns, index=index)

	# Transpose the dataframe for easier indexing.
	return df.T


def get_results(weighted_values, ranking_values):
	""" Print ranking of values. Returns dict: values_dict[value] = score.
	"""
	# Fill mx dictionary. mx[score] = [values with that score]
	mx = {}
	for i in range(len(ranking_values)):
		if weighted_values[i] not in mx.keys():
			mx[weighted_values[i]] = []
			mx[weighted_values[i]].append("v" + str(i))
		else:
			mx[weighted_values[i]].append("v" + str(i))

	# Create a list of tuples ordered by score.
	s = sorted(list(mx.items()))

	# Create string to be printed.
	string_out = ' ~ '.join(s[0][1])
	for i in range(len(s)-1):
		string_out += " < " + ' ~ '.join(s[i+1][1]) + ' '

	# Store values and scores in a dict: values_dict[value] = score
	values_dict = {}
	for key, values in mx.items():
		for value in values:
			values_dict[value] = key

	# Return sorted list of scores.
	return [values_dict[value] for value in value_list], string_out
